##############################################################################
# CustomClass Rules                                                          #
# =================                                                          #
#                                                                            #
#1. All custom classes must inherit sreb.EBObject and the constructor        #
#   must call sreb.EBObject's constructor. If a class starts with _ then it  #
#   is considered internal and will not be treated as a custom class.        #
#                                                                            #
#2. The custom class will only use the default constructor.                  #  
#   ie. a constructor with no parameters.                                    #
#                                                                            #
#3. To add a property provide getter and have an attribute that does not     #
#   starts with an underscore(_) or an upper case letter, and have the       #
#   attribute of a know type. Known types are int,float,str,EBPoint,EBColor, #
#   tuple, and list.                                                         #
#   If an attribute's default value is of type tuple and only has two        #
#   items, the property will be treated as an EBPoint. Similarly, if an      #
#   attribute's default value is a tuple of 3 items the property will be     #
#   treated as an EBColor.  The input type of the setter and the output type #
#   of the getter is expected to be the same as the type of the attribute.   #
#                                                                            #
#4. If only getter is provided, the property will be a readonly property.    # 
#                                                                            #
#6. The custom class may be instanciated during development to obtain        # 
#   class info. Avoid such things as display mode change, remote connections # 
#   in the constructor.                                                      #
#                                                                            # 
#7. Any method in the custom class can be called using the Execute action    #
#   By default, the return type of the method is string unless a doc string  #
#   with the following constraint is available                               #
#	a. The doc string starts with "RETURN:" (case matters)               #
#       b. Following the text "RETURN:" provide a default value of the type  #
#          or the __repr__ value of the class. eg. str for string            #
#8. If a property's setter metthod has default values for it's parameters,   #
#    the property will not accept references or equation.                    #
##############################################################################

import sreb
import sreb.time
import sreb.graphics
import pylink
import random
import math
import pypsy

class CustomClassTemplate(sreb.EBObject):
	def __init__(self):
		sreb.EBObject.__init__(self)
		self.displayScreenPath=''	# String used to get the path of the Display screen action in the expt graph
		self.screen=None		# handle of the display screen on which the drawing is performed.
		self.transparencyColor=sreb.EBColor(255,0,0)
		self.customResource=None
		self.maskLabel="MASK"
		self.foregroundLabel="FOREGROUND"
		self.backgroundLabel="BACKGROUND"
						
	# used to read and write the display screen (the screen where the drawing is done) path.
	def getDisplayScreenPath(self):
		return self.displayScreenPath	
	def setDisplayScreenPath(self,s):
		self.displayScreenPath=s	

					
	def getTransparencyColor(self):	
		return self.transparencyColor
	def setTransparencyColor(self,s):	
		self.transparencyColor=s

	def getMaskLabel(self):	
		return self.maskLabel
	def setMaskLabel(self,s):	
		self.maskLabel=s

	def getBackgroundLabel(self):	
		return self.backgroundLabel
	def setBackgroundLabel(self,s):	
		self.backgroundLabel=s

	def getForegroundLabel(self):	
		return self.foregroundLabel
	def setForegroundLabel(self,s):	
		self.foregroundLabel=s


	# this is the method used to initialize the custom class 
	# (and get the handle of the display screen for display drawing )	
	def initialize(self):
		# Get the screen handle.
		self.screen=sreb.graphics.getScreenFromPath(self.getDisplayScreenPath())	
		if self.screen is None:
			raise "EBScreen could not be accessed"
		
		# Get the screen dimension
		# Creates a custom resource object for display drawing;
		# Sets up the function pointer for display drawing and redraw-checking.
		width,height = self.screen.getSize()

	def resetTrialData(self):
		self.maskResource=None
		self.fgResource=None
		self.bkResource=None
		self.firstDisplay = True
		
		# Order the resources added to the screen matters! On the DISPLAY SCREEN editor,
		# select the resource and click on the right mouse button to bring up the popup menu
		# select the "resource order" option to adjust the order the resource appearing on the
		# display screen editor.
		# The background image should be the first resource added to the screen;
		for r in self.screen.getResources():
			if r.getLabel() == self.getForegroundLabel():
				self.fgResource=r
				self.fgResource.setVisible(False)
			elif r.getLabel() == self.getBackgroundLabel():
				self.bgResource= r
				self.bgResource.setVisible(True)
			elif r.getLabel() == self.getMaskLabel():
				self.maskResource=r
				self.maskResource.setVisible(False)	
				
		if self.fgResource is None:
			raise "Could not locate foreground resouce in screen."	
				
		if self.bgResource is None:
			raise "Could not locate foreground resouce in screen."	

		if self.maskResource is None:
			raise "Could not locate mask resource in screen."	
		
		tc=self.getTransparencyColor()
		tc=(tc[0],tc[1],tc[2])
		
		if self.customResource is not None:
			self.screen.removeResource(self.customResource)
				
		self.customResource = self.screen.createCustomResource(sreb.EBRectangle(0, 0, self.maskResource.getWidth(), self.maskResource.getHeight()), False)			
		self.customResource.fill(tc)
		self.customResource.setDrawMethodPointer(self.redraw)
		self.customResource.setShouldRedrawMethodPointer(self.shouldRedraw)			
				
		self.customResource.pygameSurface.set_colorkey(tc)	


	# This is the method used to do dispaly drawing.
	def redraw(self):
		tstart=sreb.time.getCurrentTime() 

		# do any drawing to the custom resource here
		drawingObject = self.customResource
		
		if self.firstDisplay == True:
			self.firstDisplay = False;
			self.pos = [512, 384]
		else:	
			s = pylink.getEYELINK().getNewestSample()
			if s:
				#print s
				eyeData = s.getRightEye();
				if eyeData ==None:
					eyeData = s.getLeftEye();
					
				if eyeData:
					self.pos = eyeData.getGaze()					
				
		if self.fgResource:
			x_position=int(self.pos[0]-self.maskResource.getWidth()/2)
			y_position=int(self.pos[1]-self.maskResource.getHeight()/2)

			
		if x_position is None or y_position is None:
			drawingObject.setVisible(False)
			
			self.lastRedrawTime=sreb.time.getCurrentTime()

		else:
			drawingObject.setVisible(True)	
						
			rtl=self.fgResource.getLocation()
			rtl[0]=rtl[0]-self.fgResource.getWidth()/2
			rtl[1]=rtl[1]-self.fgResource.getHeight()/2

			drawingObject.fill(self.getTransparencyColor())		
			#drawingObject.blitArea(self.fgResource, (0, 0), [x_position, y_position, self.maskResource.getWidth(),self.maskResource.getHeight()])
			drawingObject.blitArea(self.fgResource, (0, 0), [x_position-rtl[0], y_position-rtl[1],self.maskResource.getWidth(),self.maskResource.getHeight()])

			drawingObject.blit(self.maskResource, (0, 0))

			drawingObject.setLocation((x_position, y_position))
			self.lastPosition=[x_position, y_position]
			self.lastRedrawTime=sreb.time.getCurrentTime()	
	
	
	# This methods is used to determine the display drawing is necessary.
	def shouldRedraw(self):
		if self.screen.getLastUpdateRetraceTime() > self.lastRedrawTime:
			return True
		return False	

